package com.akamai.hdcore.samples.utility
{
	import flash.display.DisplayObject;
	import flash.display.StageDisplayState;
	import flash.events.EventDispatcher;
	import flash.events.FullScreenEvent;
	import flash.events.NetStatusEvent;
	import flash.media.SoundTransform;
	import flash.net.NetConnection;
	import flash.net.NetStream;
	import flash.net.SharedObject;
	
	import org.openvideoplayer.components.ui.controlbar.ControlBar;
	import org.openvideoplayer.components.ui.controlbar.view.DualTimeCodeDisplay;
	import org.openvideoplayer.components.ui.controlbar.view.FullscreenButton;
	import org.openvideoplayer.components.ui.controlbar.view.GoLiveButton;
	import org.openvideoplayer.components.ui.controlbar.view.PlayPauseButton;
	import org.openvideoplayer.components.ui.controlbar.view.ScrubBar;
	import org.openvideoplayer.components.ui.controlbar.view.VolumeControl;
	import org.openvideoplayer.components.ui.shared.ControlType;
	import org.openvideoplayer.components.ui.shared.event.ControlEvent;
	
	public class BasePresenter extends EventDispatcher
	{
		
		protected var netStream:NetStream;
		protected var netConnection:NetConnection;
		protected var controlBar:ControlBar;
		protected var playPauseButton:PlayPauseButton;
		protected var scrubBar:ScrubBar;
		protected var dualTimeCodeDisplay:DualTimeCodeDisplay;
		protected var volumeControl:VolumeControl;
		protected var fullscreenButton:FullscreenButton;
		protected var goLiveButton:GoLiveButton;
		protected var sharedObject:SharedObject;
		protected var isSeeking:Boolean;
		protected var isBuffering:Boolean;
		
		private var _streamStarted:Boolean;
		private var _playArgs:Array;
		
		public function BasePresenter(controlBar:ControlBar, netStream:NetStream, netConnection:NetConnection)
		{
			this.controlBar = controlBar;
			this.netStream = netStream;
			this.netConnection = netConnection;
			setControlReferences();            
			addControlBarListeners();
			sharedObject = SharedObject.getLocal("AkamaiControlBarProperties");
			initVolume();
		}
		
		public function load(resource:String, clipStartTime:Number=NaN, clipEndTime:Number=NaN):void
		{
			_playArgs = [resource, clipStartTime, clipEndTime-clipStartTime];
		}
		
		public function destroy():void 
		{
			removeControlBarListeners();
			controlBar = null;
			netStream = null;
			netConnection = null;
			_playArgs = null;
		}
		
		public function set streamStarted(value:Boolean):void
		{
			_streamStarted = value;
		}
		
		public function get streamStarted():Boolean
		{
			return _streamStarted;
		}
		
		public function get playArgs():Array
		{
			return _playArgs;
		}
		
		public function set playArgs(value:Array):void
		{
			_playArgs = value;
		}
		
		protected function onStageFullScreen(event:FullScreenEvent):void
		{
			fullscreenButton.currentState = controlBar.stage.displayState == StageDisplayState.NORMAL ? ControlEvent.FULLSCREEN : ControlEvent.NORMALSCREEN;
		}
		
		protected function setControlReferences():void
		{
			playPauseButton = controlBar.getControl(ControlType.PLAY_PAUSE_BUTTON) as PlayPauseButton;
			scrubBar = controlBar.getControl(ControlType.SCRUB_BAR) as ScrubBar;
			dualTimeCodeDisplay = controlBar.getControl(ControlType.CURRENT_AND_DURATION_TIME_DISPLAY) as DualTimeCodeDisplay;
			goLiveButton = controlBar.getControl(ControlType.GO_LIVE_BUTTON) as GoLiveButton;
			volumeControl = controlBar.getControl(ControlType.VOLUME_CONTROL) as VolumeControl;
			fullscreenButton = controlBar.getControl(ControlType.FULLSCREEN_BUTTON) as FullscreenButton;
		}
		
		protected function addControlBarListeners():void
		{
			controlBar.stage.addEventListener(FullScreenEvent.FULL_SCREEN, onStageFullScreen);			
			controlBar.addEventListener(ControlEvent.PLAY, onPlayClick);
			controlBar.addEventListener(ControlEvent.PAUSE, onPauseClick);
			controlBar.addEventListener(ControlEvent.SEEK_BEGIN, onSeekBegin);
			controlBar.addEventListener(ControlEvent.SEEK_CHANGE, onSeekChange);
			controlBar.addEventListener(ControlEvent.SEEK_COMPLETE, onSeekComplete);
			controlBar.addEventListener(ControlEvent.VOLUME_CHANGE, onVolumeChange);
			controlBar.addEventListener(ControlEvent.FULLSCREEN, onFullscreenClick);
			controlBar.addEventListener(ControlEvent.NORMALSCREEN, onFullscreenClick);
		}
		
		protected function removeControlBarListeners():void
		{
			controlBar.stage.removeEventListener(FullScreenEvent.FULL_SCREEN, onStageFullScreen);
			controlBar.removeEventListener(ControlEvent.FULLSCREEN, onFullscreenClick);
			controlBar.removeEventListener(ControlEvent.NORMALSCREEN, onFullscreenClick);
			controlBar.removeEventListener(ControlEvent.PLAY, onPlayClick);
			controlBar.removeEventListener(ControlEvent.PAUSE, onPauseClick);
			controlBar.removeEventListener(ControlEvent.SEEK_BEGIN, onSeekBegin);
			controlBar.removeEventListener(ControlEvent.SEEK_CHANGE, onSeekChange);
			controlBar.removeEventListener(ControlEvent.SEEK_COMPLETE, onSeekComplete);
			controlBar.removeEventListener(ControlEvent.VOLUME_CHANGE, onVolumeChange);			
		}
		
		protected function onFullscreenClick(event:ControlEvent):void
		{            
		}
		
		protected function initVolume():void
		{
			setVolumeOnNetStream( (sharedObject.data.volume) ? sharedObject.data.volume : .5);
			if(volumeControl)
			{
				volumeControl.setThumbPosition(netStream.soundTransform.volume, 1);
			}
		}
		
		protected function setTimeDisplay():void
		{
		}
		
		protected function onVolumeChange(event:ControlEvent):void
		{
			setVolumeOnNetStream(event.data.value);	
			sharedObject.data.volume = event.data.value;
		}
		
		protected function onSeekBegin(event:ControlEvent):void
		{
			isSeeking = true;
		}
		
		
		protected function onSeekChange(event:ControlEvent):void
		{
			
		}
		
		protected function onSeekComplete(event:ControlEvent):void
		{
			isSeeking = false;
		}
		
		protected function onPlayClick(event:ControlEvent = null):void
		{
			if (!_streamStarted)
			{
				netStream.play.apply(this, _playArgs);
				_streamStarted = true;
			}
			else
			{
				netStream.resume();
			}
		}
		
		protected function onPauseClick(event:ControlEvent = null):void
		{
			netStream.pause();
		}
		
		protected function setVolumeOnNetStream(value:Number):void
		{
			var soundTransform:SoundTransform = new SoundTransform(value);
			netStream.soundTransform = soundTransform;
			soundTransform = null;
		}
		
	}
}
