package com.akamai.hdcore.samples.utility
{
	import flash.display.DisplayObject;
	import flash.display.Shape;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.geom.Rectangle;
	import flash.media.Video;
	
	import org.openvideoplayer.components.ui.captioning.CaptioningView;
	import org.openvideoplayer.components.ui.controlbar.ControlBar;
	import org.openvideoplayer.components.ui.debug.DebugPanelView;
	import org.openvideoplayer.components.ui.posterframe.PosterFramePlayButton;
	import org.openvideoplayer.components.ui.spinner.SpinnerView;
	
	import spark.primitives.Rect;
	
	public class MediaPlayerView extends Sprite
	{
		public static const FULLSCREEN_VIDEO_WIDTH:uint  = 1280;
		public static const FULLSCREEN_VIDEO_HEIGHT:uint = 720;
		
		private var _video:Video;
		private var _controlBar:ControlBar;
		private var _posterFrameButton:PosterFramePlayButton;
		private var _spinnerView:SpinnerView;
		private var _console:DebugPanelView;
		private var _captioningView:CaptioningView;
		private var _background:Shape
		private var _videoWidth:uint;
		private var _videoHeight:uint;
		
		private var oringinalVideoDisplayIndex:int;
		private var controlBarType:String;
		private var controlBarFactory:ControlBarFactory;
		
		public function MediaPlayerView(videoWidth:uint=640, videoHeight:uint=360, controlBarType:String = "")
		{
			if (controlBarType == "")
			{
				controlBarType = ControlBarFactory.VOD;
			}
			
			_videoWidth = videoWidth;
			_videoHeight = videoHeight;
			this.controlBarType = controlBarType;
			controlBarFactory = ControlBarFactory.getInstance();
			addEventListener(Event.ADDED_TO_STAGE, onAddedToStage);
			addEventListener(Event.REMOVED_FROM_STAGE, destroy);
		}
		
		/**
		 * 
		 * 
		 */		
		private var originalVideoRect:Rectangle 
		
		public function enterFullscreen():void
		{	
			if(_video != null)
			{
				oringinalVideoDisplayIndex = oringinalVideoDisplayIndex == 0 ? getChildIndex(video) : oringinalVideoDisplayIndex;
				
				if(originalVideoRect == null)
				{
					originalVideoRect = new Rectangle()
				}
				
				originalVideoRect.x = _video.x;
				originalVideoRect.y = _video.y;
				originalVideoRect.width = _video.width;
				originalVideoRect.height = _video.height;
				
				_video.smoothing = false;
				setChildIndex(video, numChildren-1);
			}
		}
		
		/**
		 * 
		 * 
		 */		
		public function exitFullscreen():void
		{
			if(_video != null)
			{
				_video.smoothing = true;
				_video.x = originalVideoRect.x;
				_video.y = originalVideoRect.y;
				_video.width  = originalVideoRect.width;
				_video.height = originalVideoRect.height;
				setChildIndex(video, oringinalVideoDisplayIndex);
			}
		}
		
		/**
		 * Returns the current video width that the video object is set to.
		 * @return 
		 * 
		 */		
		public function get videoWidth():uint
		{
			return _videoWidth;
		}
		
		/**
		 * Returns the current video height that the video object is set to.
		 * @return 
		 * 
		 */		
		public function get videoHeight():uint
		{
			return _videoHeight;
		}
		
		/**
		 * Returns an instance of the Video Object
		 * @return 
		 *
		 */
		public function get video():Video
		{
			return _video;
		}
		
		/**
		 * Returns an instance of the ControlBar Object
		 * @return 
		 *
		 */
		public function get controlBar():ControlBar
		{
			return _controlBar;
		}
		
		/**
		 * Returns an instance of the PosterFramePlayButton Object
		 * @return 
		 *
		 */
		public function get posterFrameButton():PosterFramePlayButton
		{
			return _posterFrameButton;
		}
		
		/**
		 *
		 * Returns an instance of the BusySpinner Object
		 * @return 
		 *
		 */
		public function get spinnerView():SpinnerView
		{
			return _spinnerView;
		}
		
		/**
		 * Returns an instance of the DebugPanelView Object
		 * @return
		 *
		 */
		public function get console():DebugPanelView
		{
			if(_console == null)
			{
				addChild(_console = getConsoleView())
				_console.bandwidthPanel.visible = false;
				_console.bufferPanel.visible = false;
				_console.streamPanel.visible = false;
				_console.bandwidthTitle.visible = false;
				_console.streamTitle.visible = false;
				_console.bufferTitle.visible = false;
				
			}
			return _console;
		}
		
		/**
		 * Returns an instance of the Background Shape Object
		 * @return 
		 * 
		 */		
		public function get background():Shape
		{
			return _background;
		}
		
		/**
		 * 
		 * @return 
		 * 
		 */		
		public function get captioningView():CaptioningView
		{
			if(_captioningView == null)
			{
				addChild(_captioningView = getCaptionView());
			}
			return _captioningView;
		}
		
		/**
		 * 
		 * @param event
		 * 
		 */		
		protected function onAddedToStage(event:Event):void
		{
			addBackgroundPanel();
			addVideo();
			addPosterFrameButton();
			addControlBar();
			addBusySpinner();
			this.removeEventListener(Event.ADDED_TO_STAGE, onAddedToStage);
		}
		
		/**
		 * 
		 * 
		 */		
		protected function addBackgroundPanel():void 
		{
			addChild(_background = getBackground());
		}
		
		/**
		 * 
		 * 
		 */		
		protected function addVideo():void
		{
			_video = new Video(_videoWidth, _videoHeight);
			_video.smoothing = true;
			_video.opaqueBackground = 0x000000;
			addChild(_video);
		}
		
		/**
		 * 
		 * 
		 */		
		protected function addControlBar():void
		{
			_controlBar = controlBarFactory.getControlBarByType(controlBarType);
			_controlBar.width = videoWidth
			_controlBar.y = _videoHeight;
			addChild(_controlBar);
		}
		
		/**
		 * 
		 * 
		 */		
		protected function addBusySpinner():void
		{
			_spinnerView = new SpinnerView();			
			_spinnerView.visible = false;
			centerDisplayObject(_spinnerView);
			addChild(_spinnerView);
		}
		
		/**
		 * 
		 * 
		 */		
		protected function addPosterFrameButton():void
		{
			_posterFrameButton = new PosterFramePlayButton(_videoWidth, _videoHeight, 100);
			centerDisplayObject(_posterFrameButton);
			addChild(_posterFrameButton);
		}
		
		/**
		 * 
		 * 
		 */		
		protected function getConsoleView():DebugPanelView
		{
			_console = new DebugPanelView(_videoWidth, _videoHeight);
			_console.visible = false;
			return console;
		}
		
		  
		/**
		 * @param event
		 * 
		 */		
		protected function destroy(event:Event):void
		{
			removeEventListener(Event.REMOVED_FROM_STAGE, destroy);
			if(_video != null)
			{
				removeChild(_video);
				_video = null;
			}
			removeChild(_controlBar);
			removeChild(_posterFrameButton);
			removeChild(_spinnerView);
			removeChild(_console);
			_controlBar = null;
			_posterFrameButton = null;
			_spinnerView = null;
			_console = null;
		}
		
		protected function getCaptionView():CaptioningView
		{
			var h:uint = 30;
			_captioningView = new CaptioningView(controlBar.width, h);
			_captioningView.y = controlBar.y - h;
			_captioningView.visible = false;
			return _captioningView;
		}
		
		
		/**
		 * 
		 * @param displayObject
		 * @private
		 */		
		private function centerDisplayObject(displayObject:DisplayObject):void
		{
			displayObject.x = (_videoWidth / 2);
			displayObject.y = (_videoHeight / 2);
		}
		
		/**
		 * 
		 * @return 
		 * @private
		 */		
		private function getBackground():Shape
		{
			var shape:Shape = new Shape();
			shape.graphics.beginFill(0x000000, 1);
			shape.graphics.drawRect(0, 0, _videoWidth, stage.stageHeight);
			shape.graphics.endFill();
			return shape;
		}
	}
}
