package
{
	import com.akamai.hd.HDNetStream;
	import com.akamai.hdcore.samples.hdn1.TokenAuthenticationPresenter;
	import com.akamai.hdcore.samples.utility.AkamaiMediaSamples;
	import com.akamai.hdcore.samples.utility.BasePresenter;
	import com.akamai.hdcore.samples.utility.BaseSample;
	import com.akamai.hdcore.samples.utility.MediaPlayerView;
	import com.akamai.logger.AkamaiConsoleLogger;
	import com.akamai.logger.AkamaiConsoleLoggerFactory;
	import com.akamai.logger.LoggerEvent;
	
	import flash.display.Sprite;
	import flash.events.ContextMenuEvent;
	import flash.events.Event;
	import flash.events.IOErrorEvent;
	import flash.events.SecurityErrorEvent;
	import flash.net.NetStream;
	import flash.net.URLLoader;
	import flash.net.URLRequest;
	import flash.utils.describeType;
	
	import org.osmf.logging.Log;
	import org.osmf.logging.Logger;
	
	[SWF(width="640", height="390", frameRate="20", backgroundColor="#1a1a1a")]
	
	/**
	 * 
	 * This sample shows how to use HDN's Primary Token service with HDCore by 
	 * requesting a token for a single stream and setting the token on HDCore's 
	 * primary token property.
	 * 
	 * This sample also utilizes Akamai's player verification technology. Therefore, 
	 * it will not play the stream past (approximately) 10 -15 seconds and will abruptly stop.  
	 * Contact CCare or your Akamai representative to find out more about player verification 
	 * implementation in your player.  
	 * 
	 */	
	public class TokenAuthenticationSample extends BaseSample
	{
		
		private static const SECURE_MP4_URL:String = AkamaiMediaSamples.SINGLE_SECURE_MEDIA_FILE;
		
		private var mediaURL:String = SECURE_MP4_URL;
		private var urlLoader:URLLoader;
		private var hostName:String;
		private var logger:AkamaiConsoleLogger;
		
		public function TokenAuthenticationSample()
		{
			addAkamaiConsoleLogger();
			requestTokenizedURL(SECURE_MP4_URL);
		}
		
		override protected function onAddedToStage(event:Event):void
		{
			super.onAddedToStage(event);
			MediaPlayerView(view).video.attachNetStream(netStream);
			HDNetStream(netStream).displayObject = this.stage
		}
		
		override protected function getView():Sprite
		{
			return new MediaPlayerView();
		}
		
		override protected function getNetStream():NetStream
		{
			return new HDNetStream(netConnection);
		}
		
		override protected function getPresenter():BasePresenter
		{
			var presenter:BasePresenter = new TokenAuthenticationPresenter(MediaPlayerView(view), HDNetStream(netStream), netConnection);
			presenter.load(mediaURL);
			toggleControls(false);
			return presenter;
		}
		
		private function requestTokenizedURL(surl:String):void
		{
			hostName = surl.substring(0, surl.indexOf("/", 9));
			makeTokenRequest(new URLRequest(getFormattedTokenServiceURL(surl)));
		}
		
		private function onURLLoaderRequestComplete(event:Event):void
		{
			var tokenizedURL:String = urlLoader.data as String;
			HDNetStream(netStream).primaryToken = tokenizedURL.split("?primaryToken=")[1];
			toggleControls(true);
			logger.debug("Token req completed and ready for sample to be played --> "+tokenizedURL.split("?primaryToken=")[1]);
		}
		
		private function toggleControls(value:Boolean):void
		{
			MediaPlayerView(view).controlBar.visible = value;
			MediaPlayerView(view).posterFrameButton.visible = value;			
		}
		
		private function onSecurityError(event:SecurityErrorEvent):void
		{
			logger.debug("securityErrorHandler: " + event.text);
		}
		
		private function onIoError(event:IOErrorEvent):void
		{
			logger.debug("ioErrorHandler: " + event.text);
		}
		
		private function getFormattedTokenServiceURL(url:String):String
		{
			var lastIndexOfHostName:uint = url.indexOf("/", 9);
			var firstIndexOfStreamParams:uint = url.indexOf("?");
			var urlWithoutHostNameOrStreamParams:String = url.substring(lastIndexOfHostName, firstIndexOfStreamParams);
			return AkamaiMediaSamples.TOKEN_SERVICE_URL + urlWithoutHostNameOrStreamParams + AkamaiMediaSamples.SECURE_USER_AND_PASSWARD;
		}

		private function makeTokenRequest(request:URLRequest):void
		{
			urlLoader = new URLLoader();
			urlLoader.addEventListener(Event.COMPLETE, onURLLoaderRequestComplete);
			urlLoader.addEventListener(SecurityErrorEvent.SECURITY_ERROR, onSecurityError);
			urlLoader.addEventListener(IOErrorEvent.IO_ERROR, onIoError);
			urlLoader.load(request);
		}
		
		private function addAkamaiConsoleLogger():void
		{
			// Note: the logging framework used here is provisional and the API may change in future releases.
			Log.loggerFactory = new AkamaiConsoleLoggerFactory();
			logger = Log.getLogger(describeType(this).@name) as AkamaiConsoleLogger
			logger.setCallback(onLogEvent);
			AkamaiConsoleLogger.enable();
			function onLogEvent(event:LoggerEvent):void
			{
				MediaPlayerView(view).console.log(event.msg);
			}
		}
		
		override protected function onToggleTrace(event:ContextMenuEvent):void
		{
			// Note: the logging framework used here is provisional and the API may change in future releases.
			enableTrace = !enableTrace;
			AkamaiConsoleLogger.enable(enableTrace);
		}
	}
}